import sys
import socket
import code

BLACK = '\x1b[30m'
RED = '\x1b[31m'
GREEN = '\x1b[32m'
YELLOW = '\x1b[33m'
BLUE = '\x1b[34m'
MAGENTA = '\x1b[35m'
CYAN = '\x1b[36m'
WHITE = '\x1b[37m'
RESET = '\x1b[0;0m'
BOLD = '\x1b[1m'
REVERSE = '\x1b[2m'
BLACKBG = '\x1b[40m'
REDBG = '\x1b[41m'
GREENBG = '\x1b[42m'
YELLOWBG = '\x1b[43m'
BLUEBG = '\x1b[44m'
MAGENTABG = '\x1b[45m'
CYANBG = '\x1b[46m'
WHITEBG = '\x1b[47m'
CLEAR = '\x1b[2J'

class ColorizedConnection:
    __doc__ = 'insert (ansi)init strings when channel has changed from one call to an other'

    def __init__(self, connection, settings = [(RESET + WHITE), (RESET + CYAN), (BOLD + RED)]):
        """init with a socket and a list winth the init strings for the channels"""
        self.connection = connection
        self.channel = 0
        self.settings = settings



    def recv(self, size):
        """forward recv call to socket"""
        return self.connection.recv(size)



    def send(self, s, channel = 0):
        """forward send call to socket but send init first if channel has changed"""
        if (self.channel != channel):
            self.connection.send(self.settings[channel])
        self.connection.send(s)
        self.channel = channel



class StdOutTcpWrapper:
    __doc__ = 'emulate an output stream but send data over network'

    def __init__(self, connection, channel = 0, debugout = None):
        self.connection = connection
        self.name = ('<StdOutTcpWrapper::%s>' % connection)
        self.channel = channel
        self.debugout = debugout



    def write(self, s):
        if self.debugout:
            self.debugout.write((repr(s) + '\n'))
            self.debugout.flush()
        self.connection.send(s.replace('\n', '\r\n'), self.channel)



    def flush(self):
        pass


    def close(self):
        pass


class StdInTcpWrapper:
    __doc__ = 'emulate an input stream but get data from network'

    def __init__(self, connection, channel = 0, echo = 0, debugout = None):
        self.connection = connection
        self.name = ('<StdInTcpWrapper::%s>' % connection)
        self.channel = channel
        self.echo = echo
        self.debugout = debugout



    def isatty(self):
        return true



    def readone(self):
        """read one character from network"""
        while 1:
            c = self.connection.recv(1)
            if (ord(c) == 4):
                return ''
            if (ord(c) == 255):
                self.connection.recv(1)
                self.connection.recv(1)
            else:
                break
        if self.echo:
            self.connection.send(c, self.channel)
        return c



    def read(self, size = None):
        buf = ''
        while 1:
            if (size and (len(buf) >= size)):
                break
            c = self.readone()
            if (not c):
                break
            buf += c
        if self.debugout:
            self.debugout.write((repr(buf) + '\n'))
            self.debugout.flush()
        return buf.replace('\r', '')



    def readline(self, length = None):
        buf = ''
        if (not length):
            length = 1024
        while (len(buf) < length):
            c = self.readone()
            if (not c):
                break
            buf = (buf + c)
            if (c == '\n'):
                break
        if self.debugout:
            self.debugout.write((repr(buf) + '\n'))
            self.debugout.flush()
        return buf.replace('\r', '')



    def readlines(self, sizehint = None):
        l = []
        while 1:
            r = self.readline()
            if (not len(r)):
                break
            l.append(r)
        return l



    def close(self):
        pass



def remoteInteractive(port, locals = None, loop = 1, debug = 0):
    """listen on local port for a connection and present an interactive python shell. return after one usage"""
    stdin = sys.stdin
    stdout = sys.stdout
    stderr = sys.stderr
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(('', port))
    while loop:
        if debug:
            print 'Server ready...'
        s.listen(1)
        (con, addr,) = s.accept()
        if debug:
            print 'Connected by', addr
        con.send(CLEAR)
        col = ColorizedConnection(con)
        if debug:
            sys.stdin = StdInTcpWrapper(col, channel=0, echo=1, debugout=stdout)
            sys.stdout = StdOutTcpWrapper(col, channel=1, debugout=stdout)
            sys.stderr = StdOutTcpWrapper(col, channel=2, debugout=stdout)
        else:
            sys.stdin = StdInTcpWrapper(col, channel=0, echo=1)
            sys.stdout = StdOutTcpWrapper(col, channel=1)
            sys.stderr = StdOutTcpWrapper(col, channel=2)
        try:
            code.interact(local=locals)
        except SystemExit:
            pass
        con.close()
    sys.stdin = stdin
    sys.stdout = stdout
    sys.stderr = stderr


if (__name__ == '__main__'):

    def help(o):
        print o.__doc__


    remoteInteractive(23, locals=vars())
    sys.exit(0)
